/**
 * Created by jmoutard on 21/07/16.
 */

'use strict';

angular.module('adf.electre')
    .controller('navigationCtrl', [
        "$scope", "$rootScope", "$q", "$location", "storeService", "defaultDashboard", "$timeout", "loggedIn",
        function ($scope, $rootScope, $q, $location, storeService, defaultDashboard, $timeout, loggedIn) {
            /**
             * @description Display errors from the storage service.
             * @param {object} error Error from the storage service.
             */
            function displayError (error) {
                $scope.errorMessage = (error.message ? error.message() : error);
                $timeout(function () {
                    $scope.errorMessage = false;
                }, 2500);
            }
            /**
             * @description Taking the dashboards from the storage. Creating objects for the navbar menu.
             * @param {Array} values Data given by the storage.
             */
            function takeDashboards (values) {
                if (values && typeof values === "object") {
                    if (loggedIn) {
                        nav.items = values;
                    } else {
                        nav.items = values;
                    }
                }
            }
            /**
             * @description Removing all dashboard's object in order to clear the navbar menu.
             */
            function resetDashboards () {
                nav.localDashboard = false;
                nav.serverDashboard = false;
                $scope.serverDashboardTree = undefined;
                $scope.localItem = undefined;
            }

            var nav = this;
            nav.navCollapsed = false;
            nav.items = [];

            this.toggleNav = function(){ nav.navCollapsed = !nav.navCollapsed; };

            this.navClass = function(page) {
                var currentRoute = $location.path().substring(1);
                return page === currentRoute || new RegExp(page).test(currentRoute) ? 'active' : '';
            };

            /**
             * @description Create a new dashboard and save it.
             */
            this.create = function(){
                var created_dashboard = JSON.parse(JSON.stringify(defaultDashboard)), q;
                created_dashboard.id = '_' + Date.now();

                if (loggedIn) q = storeService.add(created_dashboard.id, created_dashboard, true);
                else          q = storeService.add(created_dashboard.id, created_dashboard, false);

                q.then(function(){
                    $location.path((loggedIn ? '/remote_board/' : '/local_board/') + created_dashboard.id);
                    $rootScope.$broadcast('navChanged');
                });
            };

            storeService.getAll().then(takeDashboards);

            $scope.$on('navChanged', function(){
                resetDashboards();
                storeService.getAll().then(takeDashboards);
            });
            $scope.$on('updateDashboard', function (event, id, data) {
                storeService.add(id, data).then(function () {
                    $scope.$emit('navChanged');
                }, function (error) {
                    displayError(error);
                });
            });
            $scope.$on('removeDashboard', function (event, id) {
                storeService.remove(id)
                    .then(function () {
                        $scope.$emit('navChanged');
                        $location.path('/');
                    }, function (error) {
                        displayError(error);
                    });
            });

            /* ROUTING */
            $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
                var isDashboardNotSaved = {answer:true};
                $rootScope.$broadcast("isDashboardSaved", isDashboardNotSaved);
                if (!isDashboardNotSaved.answer)
                if (fromState.name === "local_board" || fromState.name === "remote_board")
                if (!confirm("Dashboard not saved ! Are you sure you want to leave without saving the dashboard ?"))
                    event.preventDefault();
            });
            $rootScope.$on('$stateChangeError', function (event, toState, toParams, fromState, fromParams, error) {
                console.log(event, toState, toParams, fromState, fromParams, error);

            });
            $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
                if (toState.name === 'logout') {
                    $rootScope.$broadcast('navChanged');
                    $location.path('/');
                }
            });
        }
    ]);